##
## USED GLOBALS: env=env.base, our base environment, which for working purposes we
##               designated it to be the .GlobalEnv.  It is now the environment for function main()
##               that wraps up all programs and functions.
##               Also, env=env.who (an environment), which we create to stores all
##               static variables to be shared by all functions and programs.  
##               We assign env.who to the env.base as, assign("env.who", env=env.base)
##               if you want to see what is in env.who:  >  ls(env=get("env.who", env=env.base)); 
##
## DESCRIPTION:  The programs static variables, are given default values with
##               the function WHO(); variables are stored in the environment env.who,
##               which belongs to env.base. Any variables belonging to env.who,
##               may be retreived from env.base as:
##               get("whodisease",env=get("env.who", env=env.base)); 
##               and you may change its value as
#               assign("whodisease",newdisease, env=get("env.who", env=env.base))
##
## FORMAT: WHO(ebase=env.base) 
##
## VALUE: all variables used in the programs with their default values in WHO()
##
## WRITTEN BY: Elena Villalon & Federico Girosi
##             evillalon@latte.harvard.edu,fgirosi@latte.harvard.edu
##             CBRSS, Harvard University
##
## LAST MODIFIED: 10/27/2003
## 
## 
## ************************************************************************
## ************************************************************************

### env.base is the environmnet for function main, which may be globalized 
### assign("env.base", env.base, env= get(".GlobalEnv", env=.GlobalEnv))
##ebase = get("env.base", env=.GlobalEnv)

WHO <- function(ebase=NULL){
### we are not attaching ewho to env.base but returning it
### because we need to make it available to stand alone functions.
###  ebase <- get("env.base", env=parent.frame())
### find the environment to store variables
### the parent.frame() is env.base (driver environment),
### but if driver does not exist then default is .GlobalEnv

if(length(ebase) <= 0){
  ebase <- get("env.base", env=parent.frame())}
                            
  env.who <- get("env.who", env= ebase)         

                                              
  input.lst <- ls(env=env.who)
  n.input <-  length(input.lst)
 
    for(i in 1:n.input){
   
      val <- NULL
      val <- try(get(input.lst[i], env=env.who,inherits=T), silent=F)
      chk.env <- (input.lst[i] == "env.who" ||  input.lst[i] == "ewho" ||
                  input.lst[i] == "save.GUI" || input.lst[i] == "eGUI")
      chk.combo <- (trim.blanks(input.lst[i]) == "comboEntryCov" ||
                    trim.blanks(input.lst[i]) == "comboEntryType" ||
                    trim.blanks(input.lst[i]) == "comboValCov" ||
                    trim.blanks(input.lst[i]) == "comboValType" ||
                    trim.blanks(input.lst[i]) == "comboEntry" ||
                    trim.blanks(input.lst[i]) == "comboValue" )

      
      if(class(val) != "try-error"&& !is.environment(val)
         && length(val) > 0){
        assign(input.lst[i], get(input.lst[i],
                                 env=env.who, inherits=T),environment())
      }else if(!(chk.env || chk.combo))
        cat(paste("Cannot find the global ", input.lst[i], "\n"))
                            
    }                          

### Enter all variables as vectors; such as whousercntrylist <- c(2450, 4280)
### cause of death we want to predict
ewho <- env.who
whodisease <- get("depvar",env=ewho)

### gender: 2 for males, 3 for females
whogender <- get("strata", env=ewho)

### whocovariates: list of covariates to use.  At the moment the possible covariates
### are: gdp,hc,tfr.  Putting the prefix "ln" in front of the name of a covariate
### causes the algorithm to log that covariate.

whocovariates <-  get("cov", env=ewho)

### the last year of the out of sample period
whofore <- get("fore",env=ewho)

### the last year of the insample period
whoyrest <- get("yrest",env=ewho)

### the lag of the covariates
wholag <- get("lag",env=ewho)

### The data available for covariates and dth expand a time series,
### which may not start at the same year. The maximum years, wholag.max,
### we may lag the covariates without neglecting any mortality data,
### is calculated after substracting the starting year of observation
### for any not missing covariates, and the first year of dth observation.
### The max years depend on any specific csid (cntry+age.group).
### For predictions beyond wholag.max use only "cnts" and "time".  
###
### maximum year of available data (say, beyond 2000).  
### whofore = whoyrest + wholag.max, with covariates data 
### usually, wholag.max <- 30
### otherwise you may only use as covariates
### if(wholag > wholag.max )
###  whocovariates <- c("cnst", "time")


### whousercntrylist: A list of country codes, used to select to set
### of countries you wish to analyze. If NA the program will look
### for whoskip

whousercntrylist <- get("usercntrylist", env=ewho)
### whousercntrylist <- c(1430,2450,2020,2090,2310,4080,4085,4180,4280,3160,4308);

### this is the list of covariates that we want to exclude from the age groups
      
### specified by who.age.select
who.cov.select <- get("cov.select",env=ewho)

########################### DEFINE STRUCTURES OF cstsid ###############################
###

 who.cntry.digits <- get("cntry.digits", env=ewho)
 who.year.digits  <- get("year.digits", env=ewho)
 who.digit.first  <- get("digit.first", env=ewho)


### the covariates specified in who.cov.select will be deleted from the
### age groups specified in who.age.select
  who.age.digits <- get("age.digits", env=ewho)
  who.age.select <- get("age.select",env=ewho)


########### END OF DEFINE STRUCTURES cstsid #########################

### the percentage of data we are willing to lose in order to include the covariates
### with missing values
### who.lag.cutoff <- 0.5 
who.lag.cutoff <- get("lag.cutoff", env=ewho)
  
### whoskip: skip all countries with less than whoskip observations
### in the insample period. If NA or 0 this option is not
### used. This option is used only if both whousercntrylist and
### whousersubreg are NA
whoskip <- get("skip", env=ewho)

### select specfic age groups (they must be adjacent to each other)
#whouserages <- c(20,25,30,35);
whouserages <- get("userages", env=ewho)
###whouserages <- c(20,25,30,35);
  
### if TRUE the program will attempt to re-utilize the global variables
### computed in the preprocessing stage by make.mortality.data and stored
### in a data file whose name matches cause of death and gender and
### is stored in the directory who.reuse.dir. If the
###+ data file does not exist in the directory whooutpath, a warning
### message is printed and the pre-processing function is invoked. If the
### data file exists a number of tests are run to make sure that the
### user-set global variables stored in the data file exactly match the
### user-set global variables in this file. If an exact match is not
### found a warning message is printed and the pre-processing function
### make.mortality.data is invoked.
 
who.reuse.data <- get("reuse.data",env=ewho)
  
### The output of the simulation may be saved into a file,
### if who.save.out.put=T; it is the return object (list)
### of mortality.driver, which is the driver of the simulation
  
who.save.output <- get("save.output",env=ewho)
### The files for the covarites may be saved in the long format,
### which includes all age groups and strata values after reading
### the covarites' files either strata.age.independent, age.strata.independent, 
### age.independent, strata.independent and put into the long format and save

### the name of the directory where to look for data when who.reuse.data = TRUE
### usually this is the same as whooutpath
who.reuse.path <- get("reuse.path", env=ewho)

### which model you want to use. Current options are "OLS", "LC", "POISSON". If not a string
### no forecast is produced, and the program only creates the data. 
whomodel <- get("model",env=ewho)

### if 1 the covariates (as stored in whocov) are standardized to zero mean and std 1
whostandardize <- get("standardize", env=ewho)

### if 1 during the preprocessing we search for collinearities and delete redundant
### covariates if necessary
whoelim.collinear <- get("elim.collinear", env=ewho) 

### transform the dependent variable with function f(x): if 1, f(x) = log(x),
### if 2, f(x) = sqrt(x) and if 3, f(x) = log(1+x), if NA or 0, then f(x) = x
whotransform <- get("transform", env=ewho)

### directory where mortality data are
datapath <-  get("data.path",env=ewho)
whodatapath <- datapath

### directory where covariates are
whocovpath <- get("cov.path", env=ewho)
 
### directory where priors are  
whopriorpath <- get("prior.path", env=ewho)
  
### directory where output files will be saved
whooutpath <- get("out.path", env=ewho)

### name of directory where graphs with forecasts should be saved. if it is not a string
### then no printout is produced
whographdir <- get("out.path",env=ewho)


### BELOW WE HAVE QUANTITIES RELATED TO THE PRIORS
 
### If FALSE the prior  does not have zero mean
### and it is centered around who.mean.age.profile. Having a non-zero mean is achieved by
### subtracting the mean from the dependet variable and adding it back after the forecast
### has been done. If TRUE no action is taken.
who.zero.mean <- get("zero.mean", env=ewho)

############################ CHOICES FOR OLS ##################################
who.ols.sigma.param <- list(sigma.bar=1,use.deaths=FALSE,average=TRUE,model.based=FALSE)

############################ PRIOR OVER AGE GROUPS  ###############################

###
who.Ha.deriv <- get("Ha.deriv",env=ewho);

### If 0 or NA then all age groups are weighted equally. If scalar, then the weight
### of age group a is proportional to a^who.Ha.age.weight. If vector of length A
### then it is taken as the weight vector.
who.Ha.age.weight <- get("Ha.age.weight",env=ewho);
who.Ha.time.weight <- get("Ha.time.weight",env=ewho);

### the average standard deviation of the prior. If NA the prior is not used
### (it is like having an infinite standard deviation 
who.Ha.sigma <- get("Ha.sigma", env=ewho)
who.Ha.sigma.sd <- get("Ha.sigma.sd", env=ewho)

############################ PRIOR OVER AGE AND TIME GROUPS  ###############################

### the "age part" of the age/time prior
who.Hat.a.deriv <- get("Hat.a.deriv", env=ewho);

### the "time part" of the age/time prior
who.Hat.t.deriv <- get("Hat.t.deriv",env=ewho);

### If 0 or NA then all age groups are weighted equally. If scalar, then the weight
### of age group a is proportional to a^who.Ha.age.weight. If vector of length A
### then it is taken as the weight vector.
who.Hat.age.weight <- get("Hat.age.weight", env=ewho);

who.Hat.time.weight <- get("Hat.time.weight", env=ewho);

who.Hat.sigma <- get("Hat.sigma", env=ewho)
who.Hat.sigma.sd <- get("Hat.sigma.sd", env=ewho)

############################ PRIOR OVER  TIME GROUPS  ###############################

### 
who.Ht.deriv <- get("Ht.deriv", env=ewho)

who.Ht.age.weight <- get("Ht.age.weight", env=ewho)

who.Ht.time.weight <- get("Ht.time.weight",env=ewho)

who.Ht.sigma <- get("Ht.sigma", env=ewho)
who.Ht.sigma.sd <- get("Ht.sigma.sd", env=ewho)
                                       
############################ PRIOR OVER CNTRY GROUPS  ###############################
who.Hct.t.deriv <- get("Hct.t.deriv", env=ewho); ### smooths trend over cntry's
### who.Hct.t.deriv <- c(1)  ### smooths mortality
### we do not offer the option of setting who.Hct.time.weight != NA
### because it gets messy when different countries have different
### length of time series.
### Uniform weighting is the only option we have now
### Not implemented in the code                  
##who.Hct.cntry.weight <- get("Hct.cntry.weight", env=ewho)
##
who.Hct.c.deriv <- get("Hct.c.deriv", env=ewho)
who.Hct.sigma   <-  get("Hct.sigma", env=ewho)
who.Hct.sigma.sd <- get("Hct.sigma.sd", env= ewho)
who.Hct.time.weight <- get("Hct.time.weight", env=ewho)

  env.res <- environment()
return(env.res)
} ## end WHO()


input.lst <- list("whodisease", "whogender","whocovariates","cov.type", "whofore","whoyrest",   
                  "wholag", "wholagdth","whogdp2","whodepvar","whousercntrylist",
                  "who.cov.select","who.cntry.digits", 
                  "who.year.digits","who.digit.first","who.age.digits","who.age.select", 
                  "who.lag.cutoff","whousersubreg","whoskip", "whouserages", "who.reusedata", 
                  "who.reuse.path","who.save.output","whomodel", "whostandardize", "whoelim.collinear",
                  "whotransform", "datapath", "whodatapath", "whocovpath","whooutpath",
                  "whographdir", "tol", "svdtol", "who.prior.path", "who.zero.mean", 
                  "who.ols.sigma.param", "who.Ha.deriv", "who.Ha.age.weight", 
                  "who.Ha.time.weight", "who.Ha.sigma","who.Ha.sigma.sd",
                  "who.Hat.a.deriv","who.Hat.t.deriv",
                  "who.Hat.age.weight", "who.Hat.time.weight",
                  "who.Hat.sigma", "who.Hat.sigma.sd","who.Ht.deriv", 
                  "who.Ht.age.weight", "who.Ht.time.weight",
                  "who.Ht.sigma", "who.Ht.sigma.sd","who.Hct.t.deriv","who.Hct.time.weight",
                  "who.Hct.c.deriv","age.groups", "who.Hct.sigma","who.Hct.sigma.sd",
                  "LI.sigma.mean","LI.sigma.sd", "nsample")
###

check <- function(input = input.lst, ebase=env.base){
         ebase <- get("env.base", env=parent.frame());
          ewho <- get("env.who", env=ebase)
          ls(env=ewho)
          input <- unlist(input)
          vec <- ls(env=ewho)
          i1 <- match("ebase", vec)
          i2 <- match("env.who", vec)
          if(i1 > i2){
            vec <- vec[-i1]
            vec <- vec[-i2]
          }else{
            vec <- vec[-i2]
            vec <- vec[-i1]}

          indx <- sapply(input, function(y, ebase) {match(y,vec)},ebase)
          return(indx)}
                     
